// Praktikum Angewandte Informatik
// Reversi

// Defines

#define SIZE_X 8
#define SIZE_Y 8

#include <iostream>
#include <Reversi_KI.h>
#include <Reversi_KI_greedy.h>

using namespace std;

/** gibt das Spielfeld auf die Konsole aus.
 *
 */
void spielfeld_anzeigen(int spielfeld[SIZE_X][SIZE_Y])
{
    cout << "  ";

    for (int j=65;j<65+SIZE_Y;j++)
    {
        cout << ((char) j) << " " ;
    }
    cout <<  endl;
    for (int i=0;i<SIZE_X;i++)
    {
        cout << i+1;

        for (int j=0;j<SIZE_Y;j++)
        {
            switch (spielfeld[j][i])
            {
            case 0:
                cout << "  " ;
                break;

            case 1:
                cout << " X";
                break;

            case 2:
                cout << " O";
                break;

            default:
              cout << "Inconsistent data in field!" << endl;
              cout << "Aborting .... " << endl;
              //exit(0);
              break;
            }
        };//for j

        cout << endl;
    }//for i
}

/** \brief ermittelt, wer der Gewinner ist.
 *
 * prüft für jedes Feld des spielfeld-Arrays, welchem Spieler der
 * darauf liegende Stein gehört. Inkrementiert entsprechend zwei Zähler
 * und entscheidet, wer gewonnen hat.
 *
 */
int gewinner(int spielfeld[SIZE_X][SIZE_Y])
{
    int count_p1=0;
    int count_p2=0;

    for (int i=0;i<SIZE_X; i++)
    {
        for (int j=0;j<SIZE_Y; j++)
        {
            //loop through all fields + counting of X (1) and O (2)
        	if (spielfeld[i][j] == 1)
        	{
				count_p1++;
        	}
        	else if (spielfeld[i][j] == 2) count_p2++;
        }
    }
    if (count_p1 == count_p2)
    {
        return 0;
    }
    if (count_p2 > count_p1)
    {
        return 2;
    }
    else
    {
        return 1;
    }
}

/** \brief überprüft für übergebene Koordinaten, ob ein Zug auf dieses Feld möglich ist.
 *
 * prüft die direkte Umgebung der eingegebenen Koordinaten. Wenn dort ein
 * gegnerischer Stein gefunden wird, wird in der entsprechenden Richtung so lange weiter
 * gesucht, bis ein eigener Stein oder der Spielfeldrand gefunden wird.
 * Bricht ab, sobald EINE gültige Möglichkeit gefunden wird, gegnerische Steine einzuschließen.
 *
 */
bool zug_gueltig(int spielfeld[SIZE_X][SIZE_Y], int spieler, int pos_x, int pos_y)
{
    // Process all possible directions 
    int gegner=-(spieler -2) + 1;// the same as: if spieler=1 -> spieler=2 else
                                 // if spieler=2 -> spieler=1
    int cursor_x = 0;
    int cursor_y = 0;

    if (spielfeld[pos_x][pos_y]!=0)//check if field is currently empty
    {
        return false;
    }

    for (int i=-1;i<=1;i++)
    {
        for (int j=-1;j<=1;j++)
        {
            //check if you can find an opponents stone in a neighboring field
            //then check if in this direction all stones are opponent stones until
            //the line is terminated by one of your own stone
            //in that case return true otherwise not
        	if (pos_x + i < 0 || pos_x + i > 7
        			|| pos_y + j < 0 || pos_y + j > 7) continue; // Überspringt ungültige Felder

        	if (spielfeld[pos_x + i][pos_y + j] == gegner)
        	{
        		// gegnerischer Stein gefunden. Position merken.
				cursor_x = pos_x + i;
				cursor_y = pos_y + j;
				while (spielfeld[cursor_x][cursor_y] == gegner // noch gegnerischer Stein
						&& (cursor_x <= 7 && cursor_x >= 0)		// links/rechts rausgelaufen
						&& (cursor_y <= 7 && cursor_y >= 0))	// oben/unten rausgelaufen
				{
					// Cursor weiterschieben.
					cursor_x += i; cursor_y += j;
				}
				// nächster Stein ist kein gegnerischer mehr
				if (spielfeld[cursor_x][cursor_y] == spieler) return true;
        	}
        }
    }
    return false;
}

/** \brief führt einen Zug auf die angegebenen Koordinaten aus.
 *
 * Vorgehen ähnlich wie bei zug_gueltig, wobei hier in alle Richtungen geprüft wird,
 * ob gegnerische Steine eingeschlossen sind. Alle eingeschlossenen Steine wechseln die Farbe.
 *
 */
bool zug_ausfuehren(int spielfeld[SIZE_X][SIZE_Y], int spieler, int pos_x, int pos_y)
{
    // very similar to function "zug_gueltig" - just take care that all opponent
    // stones are changed to yours
	// Process all possible directions
	    int gegner=-(spieler -2) + 1;// the same as: if spieler=1 -> spieler=2 else
	                                 // if spieler=2 -> spieler=1
	    int cursor_x = 0;
	    int cursor_y = 0;
	    bool successFlag = false;

	    if (spielfeld[pos_x][pos_y]!=0)//check if field is currently empty
	    {
	        return false;
	    }

	    spielfeld[pos_x][pos_y] = spieler;

	    for (int i=-1;i<=1;i++)
	    {
	        for (int j=-1;j<=1;j++)
	        {
	            //check if you can find an opponents stone in a neighboring field
	            //then check if in this direction all stones are opponent stones until
	            //the line is terminated by one of your own stone
	            //in that case return true otherwise not
	        	if (pos_x + i < 0 || pos_x + i > 7
	        	        || pos_y + j < 0 || pos_y + j > 7) continue; // Überspringt ungültige Felder

	        	if (spielfeld[pos_x + i][pos_y + j] == gegner)
	        	{
	        		// gegnerischer Stein gefunden. Position merken.
					cursor_x = pos_x + i;
					cursor_y = pos_y + j;
					while (spielfeld[cursor_x][cursor_y] == gegner // eigener Stein gefunden
							&& (cursor_x + i <= 7 && cursor_x + i >= 0)		// links/rechts rausgelaufen
							&& (cursor_y + j <= 7 && cursor_y + j >= 0))	// oben/unten rausgelaufen
					{
						spielfeld[cursor_x][cursor_y] = spieler;
						// Cursor weiterschieben.
						cursor_x += i; cursor_y += j;
					}
					if (spielfeld[cursor_x][cursor_y] == spieler) successFlag = true;
					else // Rückstellen, falls ein Fehlzug gemacht wurde.
					{
						while (cursor_x != pos_x+i || cursor_y != pos_y+j)
						{
							cursor_x -= i; cursor_y -= j;
							spielfeld[cursor_x][cursor_y] = gegner;
						}
					}
	        	}
	        }
	    }
return successFlag;
}

/** moegliche_zuege geht das Spielfeld durch und prüft für jede Koordinate,
 * ob ein Zug auf diese möglich ist. In diesem Fall wird ein Zähler inkrementiert.
 *
 */
int moegliche_zuege(int spielfeld[SIZE_X][SIZE_Y], int spieler)
{
	int counter = 0;

	for (int i=0;i<=SIZE_X;i++)
		    {
		        for (int j=0;j<=SIZE_Y;j++)
		        {
		        	if (zug_gueltig(spielfeld,spieler,i,j)) counter++;
		        }
		    }
	return counter;
}

/** \brief fragt über die Konsole den Zug eines menschlichen Spielers ab.
 *
 */
bool mensch_zug(int spielfeld[SIZE_X][SIZE_Y], int spieler)
{
    if (moegliche_zuege(spielfeld,spieler)==0)
    {
        return false;
    }
    int px;
    int py;
    bool repeat=false;

    do
    {
        if (repeat)
        {
            cout << endl << "Invalid input !" << endl;
        }
        string eingabe;
        eingabe.reserve(50);
        cout << endl << "Your move (e.g. A1): " ;
        eingabe.erase(eingabe.begin(),eingabe.end());
        cin >> eingabe;
        px = ((int) eingabe.at(0) ) - 65;
        py = ((int) eingabe.at(1) ) - 49;
    }
    while (repeat=(!zug_gueltig(spielfeld,spieler,px,py)));
    zug_ausfuehren(spielfeld,spieler,px,py);
    return true;
}


void spiel(int spielertyp[2])
{
  int spielfeld[SIZE_X][SIZE_Y]={
      {0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,0,0},
      {0,0,0,1,2,0,0,0},
      {0,0,0,2,1,0,0,0},
      {0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,0,0}};

  int aktueller_spieler=1;	// enthält den Spieler, der aktuell am Zug ist.
  bool zugBeendet = false;	// dient als Flag, ob der Zug ausgeführt ist.
  bool schwimmen[2] = {false, false};	// dient als Flag, ob ein Spieler aussetzt.
  int moveCounter = 0;	// zählt die ausgeführten Züge

  spielfeld_anzeigen(spielfeld);
  //let each player make its moves until no further moves are possible
  while (moegliche_zuege(spielfeld,aktueller_spieler) > 0	// solange noch Züge möglich sind...
		  || schwimmen[aktueller_spieler-1] == false)		// ...oder der aktuelle Spieler ausgesetzt hat
  {
	  if(moegliche_zuege(spielfeld,aktueller_spieler) == 0) // Spieler kann nicht ziehen und muss aussetzen.
	  {
		  schwimmen[aktueller_spieler-1] = true;
		  aktueller_spieler=-(aktueller_spieler -2) + 1; // Spieler wechseln.
		  continue;
	  }

	  schwimmen[aktueller_spieler-1] = false; //resettet das Aussetzen-Falg.

	  cout << "I'ts Player " << aktueller_spieler <<"'s turn." << endl;

	  if(spielertyp[aktueller_spieler-1] == 1) // unterscheiden, ob ein Mensch oder ein Computer spielt.
	  {
		  zugBeendet = mensch_zug(spielfeld, aktueller_spieler);
	  }
	  else if (spielertyp[aktueller_spieler-1] == 2)
	  {
		  zugBeendet = computer_zug(spielfeld, aktueller_spieler);
	  }
	  else zugBeendet = computer_zug_greedy(spielfeld, aktueller_spieler);

	  aktueller_spieler=-(aktueller_spieler -2) + 1; // Spieler wechseln.

	  moveCounter++;
	  cout << "---------------------------------------------\n"
			  << "Situation after move no. " << moveCounter << endl;
	  spielfeld_anzeigen(spielfeld); // Spielfeld ausgeben
	  cout << endl;

  }

  cout << "Spiel beendet. ";
  switch (gewinner(spielfeld))
  {
  case 0: cout << "Unentschieden." << endl; break;
  case 1: cout << "Spieler 1 gewinnt." << endl; break;
  case 2: cout << "Spieler 2 gewinnt." << endl; break;
  }
}


int main(void)
{
    /*int spielfeld[SIZE_X][SIZE_Y]={
        {0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0},{0,0,0,1,2,0,0,0},
        {0,0,0,2,1,0,0,0},{0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0}};

   spielfeld_anzeigen(spielfeld);
    */

    int typ[2] = { 2, 2 };
    cout << "### REVERSI ###\n"
    		<< "Is Player 1 a human (1) or a computer (2) or a greedy computer (3)?\n";
    cin >> typ[0];
    cout << "Is Player 2 a human (1) or a computer (2) or a greedy computer (3)?\n";
    cin >> typ[1];
    cout << "Understood. Starting game..." << endl;
    sleep(2);
    cout << "---------------------------------" << endl;
    spiel(typ);
    return 0;

}

