//////////////////////////////////////////////////////////////////////////////
// Praktikum Informatik 1
// Versuch 7: L�sung eines mathematischen Anwendungsproblems
//
// Datei:  Cramer.cpp
// Inhalt: Cramer-Klasse
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
///////////////////////////////// INCLUDES ///////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#include "Cramer.h"
#include "QMatrix.h"
#include "Vektor.h"
#include <cmath>

//////////////////////////////////////////////////////////////////////////////
///////////////////////// Funktions-Implementierung //////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Konstruktor
Cramer::Cramer(int n)
{
	A_invers = new QMatrix(n);
	x = new Vektor(n);
}

// Destruktor
Cramer::~Cramer()
{
	delete A_invers;
	delete x;
}

Vektor Cramer::loese(QMatrix A, Vektor b)
{
	invertiere(A);
	int dimension = A.getDim();

	// x = A_invers * b
	for (int i = 0; i < dimension; ++i)
	{
		double temp = 0;
		for (int j = 0; j < dimension; ++j)
		{
			temp += A_invers->get(i,j) * b.get(j);
		}
		x->set(i, temp);
	}

	return *x;
}

// berechnet die Determinante der Matrix A
double Cramer::determinante(QMatrix& A)
{
	if(A.getDim() == 1) return A.get(0,0); // Rekursionsausstieg

	double result = 0;
	int dimension = A.getDim();

	for (int i = 0; i < dimension; ++i)
	{
		QMatrix unter_A = A.untermatrix(0, i);

		result += (i%2 ? -1 : 1) * A.get(0,i) * determinante( unter_A );
	}
	return result;
}

// invertiert die Matrix A und speichert das Ergebnis in A_invers
void Cramer::invertiere(QMatrix& A)
{
	int dimension = A.getDim();
	double det_A = determinante(A);

	// Adjunkte berechnen, sofort durch die Determinante teilen.
	for (int i = 0; i < dimension; ++i)
	{
		for (int j = 0; j < dimension; ++j)
		{
			QMatrix unter_A = A.untermatrix(j,i);
			A_invers->set(j,i,// umdrehen, um sofort zu transponieren
					((i+j)%2 ? -1 : 1) * determinante(unter_A) / det_A
					);
		}
	}
	A_invers->transponiere();
}
